{% extends '!layout.html' %}

{% block footer %}
        <!-- https://gramps.discourse.group/t/how-can-useful-snippets-be-proposed-for-the-sphinx-developer-docs/5740/4 : add to "&#169; Copyright 2001-2019, The Gramps Project. Created using <a href="http://sphinx-doc.org/">Sphinx</a> 2.0.1." -->
        &bull; <a href="https://www.gramps-project.org/wiki/index.php/Category:Developers/Reference">Become a Contributor.</a>
{% endblock %}
