/*
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2011 Rob G. Healey <robhealey1@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
#
###################################################### */
body#FamilyMap {
    background-color: #000;
    padding: 0 14px;
}

/*    Family Maps
------------------------------------------------------ */
#map_canvas {
    margin-left: 10px;
    margin-right: 10px;
    border: solid 4px #000;
    margin: 20px auto;
    width: 800px;
    height: 400px;
    max-width: 90%;
    max-height: 90%;
}

/*    Place Maps
------------------------------------------------------ */
div#place_canvas {
    margin-left: 210px;
    border: solid 4px #000;
    width: 500px;
    height: 400px;
    max-width: 90%;
    max-height: 90%;
}
button#drop {
    background-color: purple;
    color: #FFF;
    font: bold .8em sans-serif;
    padding: 10px;
    margin-top: 10px;
    margin-left: 10px;
    border: solid 4px #00029D;
}

/*    Subsection: References
------------------------------------------------------ */
div#FamilyMapDetail div#references table.infolist {
    width: 100%;
}
div#FamilyMapDetail div#references table.infolist tbody tr td.ColumnPlace {
    width: 70%;
}


/*    Subsection: popup
------------------------------------------------------ */
#map_canvas .ol-popup {
  position: absolute;
  background-color: white;
  -webkit-filter: drop-shadow(0 1px 4px rgba(0,0,0,0.2));
  filter: drop-shadow(0 1px 4px rgba(0,0,0,0.2));
  padding: 5px;
  border-radius: 8px;
  border: 2px solid #111111;
  bottom: 8px;
  left: -10px;
  min-width: 500px;
}
#map_canvas .ol-popup:after, #map_canvas .ol-popup:before {
  top: 100%;
  border: solid transparent;
  height: 0;
  width: 0;
  position: absolute;
  pointer-events: none;
}
#map_canvas .ol-popup:after {
  border-top-color: white;
  border-width: 10px;
  left: 48px;
  margin-left: -10px;
}
#map_canvas .ol-popup:before {
  border-top-color: #cccccc;
  border-width: 11px;
  left: 48px;
  margin-left: -11px;
}
#map_canvas .ol-popup-closer {
  text-decoration: none;
  position: absolute;
  top: 2px;
  right: 8px;
}
#map_canvas .ol-popup-title {
  background-color: #ededed;
  padding: 0 0.5em;
}
#map_canvas .ol-popup-content {
  color: #111;
  font: bold 0.9em sans-serif;
  max-height: 200px;
  overflow-y: auto;
  padding: 5px 0.5em 5px 0.5em;
}
#map_canvas .thumbnail {
  padding: 5px 5px;
  text-align: center;
}
#map_canvas img {
  max-height: 300px;
  max-width: 400px;
}
#map_canvas .ol-popup-content a {
  color: #111;
}
#map_canvas .ol-popup-content a:visited {
  color: green;
}
#map_canvas .ol-popup-content a:hover {
  color: red;
}
#map_canvas .ol-popup-closer:after {
  content: "✖";
}
#map_canvas .ol-control {
  position: absolute;
  background-color: rgba(255,255,255,0.1);
  bottom: .1em;
  left: .1em;
  height: 4%;
  width: 100%;
  margin-top: 1px;
  margin-bottom: 1px;
  padding-left: 1px;
}
#map_canvas .ol-rotate {
  display: none;
}
#map_canvas .ol-zoom, #map_canvas .ol-zoom-in, #map_canvas .ol-zoom-out {
  top: .5em;
  left: .5em;
  border-left-width: 1px;
  padding-left: 1px;
  padding-right: 1px;
}
#map_canvas .ol-attribution ul {
  list-style-type: none;
  font-size:0.8em;
  float: left;
  margin-top: 1px;
  margin-bottom: 1px;
  padding-left: 5px;
}
#map_canvas .ol-attribution li {
  float: left;
  padding-left: 5px;
}
#map_canvas .ol-attribution button {
    display: none;
}
